#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import with_statement

import os, sys
import locale
import hashlib
import time
import uuid
import base64
import thread
import unicodedata
from threading import Thread
from datetime import datetime

import const

SIZE_HASH = 8*1024
HOME      = const.utf(os.path.expanduser("~"), const.ENC)

# Columns order -- file_table.get_stats must be in sync with the interface
SIZE = 0
TIME = 1
MIME = 2
EXT  = 3
NAME = 4
BMD5 = 5
EMD5 = 6
FILE = 7
PARENT   = 8
FULLPATH = 9

#-------------------------------------------------------------------------------
# Another ugly workaround here: Snow Leopard old libs...
#-------------------------------------------------------------------------------
try:
    MAGIC = True
    import magic
except Exception, e:
    MAGIC = False
    import mimetypes
    const.ERROR.append(u"import_magic " + str(e.args))

#-------------------------------------------------------------------------------
def normalize(string):
    a = unicodedata.normalize('NFKD', unicode(string, errors='ignore'))
    return a.encode('utf-8', 'ignore')

#-------------------------------------------------------------------------------
def get_initial_path():
    if const.win():
        APPDATA = const.utf(os.environ['APPDATA'], const.ENC)
        path = const.utf(APPDATA + os.sep + u"Dropbox" + os.sep + u"host.db")
    else:
        path = const.utf(HOME + os.sep + u".dropbox" + os.sep + u"host.db")
    try:
        host_db = open(path) if os.path.isfile(path) else None
        if (host_db):
            host_db.readline()
            p = const.utf(base64.b64decode(host_db.readline()))
            host_db.close()
            if os.path.isdir(p) and os.access(p, os.R_OK):
                return p
            else:
                const.ERROR.append(u"get_initial_path folder_not_found")
        else:
            const.ERROR.append(u"get_initial_path host_db_not_found")
    except Exception, e:
        const.ERROR.append(u"get_initial_path " + str(e.args))
    p = const.utf(HOME + os.sep + u"Dropbox")
    return p if (os.path.isdir(p) and os.access(p, os.R_OK)) else None

#-------------------------------------------------------------------------------
def humanize_bytes(bytes, precision=0):
    abbrevs = (
        (1<<50L, u'PB'),
        (1<<40L, u'TB'),
        (1<<30L, u'GB'),
        (1<<20L, u'MB'),
        (1<<10L, u'kB'),
        (1, '')
    )
    if bytes == 1:
        return u'1'
    for factor, suffix in abbrevs:
        if bytes >= factor:
            break
    return '%.*f %s' % (precision, bytes / factor, suffix)

#-------------------------------------------------------------------------------
class file_table(Thread):
    def __init__(self, caller=None):
        Thread.__init__(self)
        self.HAS_MAGIC = MAGIC
        self.a_lock = thread.allocate_lock()
        self.path   = None
        self.stopit = False
        self.over   = True
        self.caller = caller
        self.t_str  = None
        self.t_end  = None
        self.magic  = self.open_magic()
        self.files   = 0
        self.folders = 0
        self.bytes   = 0
        self.uid     = self.get_user_id()
        with self.a_lock:
            self.logs = []

#-------------------------------------------------------------------------------
    def open_magic(self):
        if (self.HAS_MAGIC):
            f = None
            if const.win():
                f = (const.DIR + u"etc" + os.sep + \
                      u"magic_win32.mgc").encode(const.ENC)
            else:
                if const.mac():
                    f = (const.DIR + u"etc" + os.sep + \
                        u"magic_dar32.mgc").encode(const.ENC)
            m = None
            try:
                m = magic.Magic(mime=True, magic_file=f)
            except:
                self.HAS_MAGIC = False
                mimetypes.init()
                const.ERROR.append(u"open_magic " + str(e.args))
            return m
        else:
            mimetypes.init()
            return None

#-------------------------------------------------------------------------------
    def run (self):
        self.over = False
        self.get_files(self.path)
        self.update_view(True)
        if self.caller:
            self.caller.done()
        else:
            print self.prepare()
        self.over = True

#-------------------------------------------------------------------------------
    def stop(self):
        self.stopit = True

#-------------------------------------------------------------------------------
    def my_hash(self, data):
        m = hashlib.md5()
        m.update(data)
        return m.hexdigest()

#-------------------------------------------------------------------------------
    def get_log(self, row, col):
        with self.a_lock:
            if (col == SIZE):
                return humanize_bytes(self.logs[row][col])
            if (col == TIME):
                return datetime.fromtimestamp(
                        self.logs[row][col]/1000).strftime('%Y-%m-%d %H:%M:%S')
            else:
                return str(self.logs[row][col])

#-------------------------------------------------------------------------------
    def get_log_size(self):
        with self.a_lock:
            return len(self.logs)

#-------------------------------------------------------------------------------
    def get_mime_file(self, buf, path):
        mime = None
        if (self.HAS_MAGIC):
            try:
                # Try to be very fast
                mime = u"[" + self.magic.from_buffer(buf[0:256]) + u"]"
                if not mime or u" " in mime or \
                        u"application/octet-stream" in mime:
                    # Fail, go the hard way
                    mime = u"[" + self.magic.from_file(path) + u"]"
            except Exception, e:
                pass
                # No problem there will be the application/unknown in the error
                # const.ERROR.append(u"get_mime_file " + str(e.args))
        else:
            mime = "[" + mimetypes.guess_type(path, strict=False)[0] + "]"

        if not mime or u" " in mime:
            # give it up
            mime = u"[application/unknown]"
        return mime

#-------------------------------------------------------------------------------
    def get_stats(self, is_dir, p, name):
        try:
            hnam = self.my_hash(normalize(name))
            size = os.path.getsize(p)
            self.bytes += size
            time = int(1000*os.path.getmtime(p))
            parent = self.my_hash(normalize(os.path.dirname(p)))
            fullpath = "-"
            if (is_dir):
                mime = u"[application/directory]"
                exts = u"-"
                hf8k = u"-"
                hl8k = u"-"
                fullpath = self.my_hash(normalize(p))
            else:
                n, exts = os.path.splitext(p)
                if (exts == None or exts[1:] == u""):
                    exts = u"-"
                else:
                    exts = exts[1:].replace(" ", "_")
                f = open(p)
                b = f.read(SIZE_HASH)
                hf8k = self.my_hash(b)
                mime = self.get_mime_file(b, p)
                if (size > SIZE_HASH):
                    f.seek(size - SIZE_HASH)
                    hl8k = self.my_hash(f.read(SIZE_HASH))
                else:
                    hl8k  = u"-"
                f.close()
            with self.a_lock:
                self.logs.append((size, time, mime, exts, hnam, hf8k, hl8k,
                    (0 if is_dir else 1), parent, fullpath))
        except Exception, e:
            const.ERROR.append(u"get_stats " + str(e.args))

#-------------------------------------------------------------------------------
    def get_files(self, path):
        self.t_str = int(1000*time.time())
        path = path.encode(const.ENC)
        for root, dirs, files in os.walk(path, onerror=self.on_error):
            for sf in dirs:
                if (self.stopit):
                    self.over = True
                    return
                self.get_stats(True, os.path.join(root, sf), sf)
                self.folders += 1
                self.update_view()
            for f in files:
                if (self.stopit):
                    self.over = True
                    return
                self.get_stats(False, os.path.join(root, f), f)
                self.files += 1
                self.update_view()
        self.t_end = int(1000*time.time())

#-------------------------------------------------------------------------------
    def on_error(self, e):
        const.ERROR.append(u"os.walk " + str(e.args))

#-------------------------------------------------------------------------------
    def update_view(self, final = False):
        if (final or len(self.logs) in (50, 100) or len(self.logs) % 200 == 0):
            f = str(self.files)
            d = str(self.folders)
            b = humanize_bytes(self.bytes)
            self.caller.update_data(f, d, b) if self.caller else None

#-------------------------------------------------------------------------------
    def get_user_id_config(self):
        if const.win():
            try:
                import _winreg
                a = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER,
                        r'Software\Dropbox\ks', 0, _winreg.KEY_READ)
                v = _winreg.EnumValue(a, 0)[1]
                if (len(v) > 0):
                    return u"cr_" + self.my_hash(v)
                else:
                    const.ERROR.append(u"get_user_id_config empty_registry_key")
            except Exception, e:
                const.ERROR.append(u"get_user_id_config_r " + str(e))
            return None
        else:
            p = const.utf(HOME + os.sep + u".dropbox" + os.sep + u"hostkeys")
            try:
                if os.path.isfile(p):
                    return u"cf_" + self.my_hash(open(p).read())
                else:
                    const.ERROR.append(u"get_user_id_config hostkeys_not_found")
            except Exception, e:
                const.ERROR.append(u"get_user_id_config_f " + str(e.args))
            return None

#-------------------------------------------------------------------------------
    def get_user_id_mac(self):
        try:
            mac = uuid.getnode()
            return u"ma_" + self.my_hash(str(mac))
        except Exception, e:
            const.ERROR.append(u"get_user_id_mac " + str(e.args))
        return None

#-------------------------------------------------------------------------------
    def get_user_id(self):
        uid = self.get_user_id_config()
        if uid:
            return uid
        uid = self.get_user_id_mac()
        return uid if uid else u"NULL_ID"

#-------------------------------------------------------------------------------
    def prepare(self):
        lcl = locale.getdefaultlocale()
        prepared_log = u"\n".join([
            u"start " + str(self.t_str),
            u"end " + str(self.t_end),
            u"id " + self.uid,
            u" ".join([u"folder",
                self.my_hash(self.path.encode("utf-8", "ignore")), u"default",
                str(const.F_DEFAULT), str(const.F_FOUND), str(const.F_MATCH),
                str(sys.platform),  str(lcl[0]) + " " + str(lcl[1])]),
            u"Errors " + str(const.ERROR),
            u"file size time mime extension f8k_md5 l8k_md5 name_md5 " + \
                "parent fullpath\n"""])
        prepared_log += u"\n".join(
            u" ".join([str(s[FILE]), str(s[SIZE]), str(s[TIME]),
                s[MIME], s[EXT], s[BMD5], s[EMD5], s[NAME], s[PARENT],
                    s[FULLPATH]]) for s in self.logs)
        return prepared_log

#-------------------------------------------------------------------------------
if __name__ == '__main__':
    FILE_T  = file_table(None)
    FILE_T.path = get_initial_path()
    if (not FILE_T.path):
        FILE_T.path = HOME
    FILE_T.start()

