#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import wx
import wx.grid as gridlib

import wizard_1
import wizard_2
import wizard_3
import wizard_log

import const
import log_sender
import file_table

import urllib2
from urlparse import urlparse

def create(parent):
    return Wizard(parent)

[wxID_WIZARD] = [wx.NewId() for _init_ctrls in range(1)]

class Wizard(wx.Frame):
    def _init_ctrls(self, prnt):
        fs = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        if (fs):
            const.FONT = fs
            const.SIZE = fs.GetPointSize()
            if not const.win() and not const.mac():
                const.FONT_FACE = fs.GetFaceName()
            const.FONT_B = wx.Font(const.SIZE, wx.SWISS, wx.NORMAL, wx.BOLD,
                False, const.FONT_FACE)
        else:
            const.FONT = wx.Font(const.SIZE, wx.SWISS, wx.NORMAL,
                wx.NORMAL, False, u"Verdana")
            const.FONT_B = wx.Font(const.SIZE, wx.SWISS, wx.NORMAL,
                wx.BOLD, False, u"Verdana")

        wx.Frame.__init__(self, id=wxID_WIZARD, name=u'Wizard', parent=prnt,
            pos=wx.Point(180, 140), size=wx.Size(600, 380),
            style=wx.MINIMIZE_BOX | wx.SYSTEM_MENU | wx.CAPTION | wx.CLOSE_BOX,
            title=u'Dropbox crawler')

        self.SetIcon(wx.Icon(const.DIR + 'icons' +
                    os.sep + 'preferences-desktop-accessibility.png',
                    wx.BITMAP_TYPE_PNG))
        self.SetBackgroundColour(wx.Colour(240, 240, 240))
        self.SetClientSize(wx.Size(600, 380))
        self.Bind(wx.EVT_CLOSE, self.onClose)

        self.sender = None
        self.wizard_1 = wizard_1.create(self)
        self.wizard_2 = wizard_2.create(self)
        self.wizard_3 = wizard_3.create(self)
        self.wizard_2.Hide()
        self.wizard_3.Hide()

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.wizard_1, 1, wx.EXPAND)
        self.sizer.Add(self.wizard_2, 1, wx.EXPAND)
        self.sizer.Add(self.wizard_3, 1, wx.EXPAND)
        self.SetSizer(self.sizer)
        self.Layout()

    def onClose(self, event):
        if (self.FILE_T):
            self.FILE_T.stop()
        while (not self.FILE_T.over):
            pass
        self.Destroy()

    def init_params(self):
        self.INITIAL = file_table.get_initial_path()
        self.DB_HOME = self.INITIAL
        self.FILE_T  = file_table.file_table(self)

        self.wizard_2.table_base.FILE_T = self.FILE_T
        if (self.DB_HOME):
            if (len(self.DB_HOME) > 60):
                self.wizard_1.txt_folder.SetLabel(
                    self.DB_HOME[u"..."  + len(self.DB_HOME)-60:])
            else:
                self.wizard_1.txt_folder.SetLabel(self.DB_HOME)
            const.F_FOUND = True
            self.wizard_1.bt_select.Hide()
            self.wizard_1.bt_select.Disable()
        else:
            self.wizard_1.bt_start.Disable()
            const.ERROR.append(u"init_params initial_folder_not_found")

    def select_dir(self, event):
        dlg = wx.DirDialog(self, u"Choose a directory:",
            pos=wx.Point(300, 140),
            style=wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST
        )
        if (self.DB_HOME):
            dlg.SetPath(self.DB_HOME)
        if dlg.ShowModal() == wx.ID_OK:
            path = const.utf(dlg.GetPath().encode("utf-8"), const.ENC)
            if (path != u"" and os.path.isdir(path)):
                if (len(path) > 60):
                    self.wizard_1.txt_folder.SetLabel(
                        u"..." + path[len(path)-60:])
                else:
                    self.wizard_1.txt_folder.SetLabel(path)
                self.DB_HOME = path
                const.F_DEFAULT = False
                const.F_MATCH = (path == self.INITIAL)
                self.wizard_1.bt_start.Enable()
            else:
                wx.MessageBox(u'Unreadable path', u'Error',
                    wx.OK | wx.STAY_ON_TOP | wx.ICON_ERROR)
        dlg.Destroy()

    def read_dir(self, event):
        self.SetTitle(
            u"Calculating statistics - Please be patient - Dropbox crawler")
        self.wizard_1.Hide()
        self.wizard_2.Show()
        self.Layout()
        const.USE_PROXY = self.wizard_1.cb_proxy.IsChecked()
        const.PROXY_SERVER = self.wizard_1.txt_proxy_server.GetValue()
        const.PROXY_PORT = self.wizard_1.txt_proxy_port.GetValue()
        self.FILE_T.path = self.DB_HOME
        self.FILE_T.start()

    def update_data(self, files, folders, bytes):
        wx.CallAfter(self.wizard_2.table_base.update_view,
            self.wizard_2.table_data)
        wx.CallAfter(self.wizard_2.txt_files.SetLabel, files)
        wx.CallAfter(self.wizard_2.txt_folders.SetLabel, folders)
        wx.CallAfter(self.wizard_2.txt_bytes.SetLabel, bytes)

    def done(self):
        wx.CallAfter(self.wizard_2.timer.Stop)
        wx.CallAfter(self.wizard_2.progress_bar.SetValue, 0)
        wx.CallAfter(self.wizard_2.bt_log.Enable)
        wx.CallAfter(self.wizard_2.bt_send.Enable)
        wx.CallAfter(self.SetTitle, u"Ready to send - Dropbox crawler")

    def send_it(self, event):
        self.SetTitle(u"Sending - Please be patient - Dropbox crawler")
        self.wizard_2.timer.Start(100)
        self.wizard_2.table_data.Disable()
        self.wizard_2.bt_log.Disable()
        self.wizard_2.bt_send.Disable()
        self.sender = log_sender.log_sender(self, self.FILE_T)
        self.sender.daemon = True
        self.sender.start()

    def finish_sending(self):
        wx.CallAfter(self.SetTitle, u"Thanks - Dropbox crawler")
        wx.CallAfter(self.wizard_3.set_link, self.FILE_T.uid)
        wx.CallAfter(self.wizard_3.Show)
        wx.CallAfter(self.wizard_2.Hide)
        wx.CallAfter(self.wizard_2.timer.Stop)
        wx.CallAfter(self.wizard_2.progress_bar.SetValue, 0)
        wx.CallAfter(self.Layout)

    def sending_error(self):
        wx.CallAfter(self.SetTitle, u"Something went wrong - Dropbox crawler")
        wx.CallAfter(self.wizard_3.show_error)
        wx.CallAfter(self.wizard_3.Show)
        wx.CallAfter(self.wizard_2.Hide)
        wx.CallAfter(self.wizard_2.timer.Stop)
        wx.CallAfter(self.wizard_2.progress_bar.SetValue, 0)
        wx.CallAfter(self.Layout)

    def show_log(self, event):
        dlg = wizard_log.create(self)
        dlg.txt_log.SetValue(self.FILE_T.prepare())
        dlg.ShowModal()

    def __init__(self, parent):
        self._init_ctrls(parent)
        
#-------------------------------------------------------------------------------
def cb_proxy_cli(self):
    if self.cb_proxy.IsChecked():
        self.txt_proxy_port.Enable()
        self.txt_proxy_server.Enable()
        self.lb_proxy_port.Enable()
        self.lb_proxy_server.Enable()
        if (self.txt_proxy_server.GetValue() == "" and
                self.txt_proxy_server.GetValue() == ""):
            proxy_list = urllib2.getproxies()
            if proxy_list and "http" in proxy_list:
                proxy = urlparse(proxy_list["http"])
                try:
                    self.txt_proxy_server.SetValue(proxy.hostname)
                    if proxy.port:
                        self.txt_proxy_port.SetValue(str(proxy.port))
                    else:
                        self.txt_proxy_port.SetValue("80")
                except: # not a problem
                    pass
            else:
                self.txt_proxy_server.SetValue("127.0.0.1")
                self.txt_proxy_port.SetValue("8080")
    else:
        self.txt_proxy_port.Disable()
        self.txt_proxy_server.Disable()
        self.lb_proxy_port.Disable()
        self.lb_proxy_server.Disable()
        
#-------------------------------------------------------------------------------
# This functions are used by two windows in the wizard
def cb_proxy_cli_text(self):
    self.cb_proxy.SetValue(not self.cb_proxy.IsChecked())
    cb_proxy_cli(self)
    
#-------------------------------------------------------------------------------
class GenericTable(wx.grid.PyGridTableBase):
    def __init__(self):
        wx.grid.PyGridTableBase.__init__(self)
        self.rowLabels = None
        self.colLabels = [u"Size",
            u"Modified",
            u"MIME",
            u"Ext.",
            u"Name (MD5)"]
        self.FILE_T = None
        self.count = 0

    def GetNumberRows(self):
        return self.count

    def GetNumberCols(self):
        return 5

    def GetColLabelValue(self, col):
        if self.colLabels:
            return self.colLabels[col]

    def GetRowLabelValue(self, row):
        if self.rowLabels:
            return self.rowLabels[row]

    def IsEmptyCell(self, row, col):
        return False

    def GetValue(self, row, col):
        if (self.FILE_T and row < self.count):
            return self.FILE_T.get_log(row, col)
        else:
            return u""

    def update_view(self, grid):
        diff = self.FILE_T.get_log_size() - self.count
        grid.ProcessTableMessage(gridlib.GridTableMessage(
                self, gridlib.GRIDTABLE_NOTIFY_ROWS_APPENDED, diff))
        self.count = self.count + diff
        grid.AdjustScrollbars()
        grid.ForceRefresh()
